﻿using Contracts.BindingModels;
using Contracts.BusinessLogicContracts;
using Contracts.SearchModels;
using Contracts.StorageContracts;
using Contracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BusinessLogic.BusinessLogic
{
    public class ProductLogic : IProductLogic
    {
        private readonly IProductStorage _productStorage;
        private readonly ILogger _logger;
        public ProductLogic(IProductStorage productStorage, ILogger<ProductLogic> logger)
        {
            _productStorage = productStorage;
            _logger = logger;
        }
        public bool Create(ProductBindingModel model)
        {
            CheckModel(model);
            if (_productStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(ProductBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_productStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        public ProductViewModel? ReadElement(ProductSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Name:{Name}.Id:{ Id}", model.Name, model.Id);
            var element = _productStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public List<ProductViewModel>? ReadList(ProductSearchModel? model)
        {
            _logger.LogInformation("ReadList. Name:{Name}.Id:{ Id}", model?.Name, model?.Id);
            var list = model == null ? _productStorage.GetFullList() : _productStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public bool Update(ProductBindingModel model)
        {
            CheckModel(model);
            if (_productStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(ProductBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.Name))
            {
                throw new ArgumentNullException("Нет названия продукта", nameof(model.Name));
            }
            if (model.Price < 0)
            {
                throw new Exception("Цена продукта не может быть ниже нуля");
            }
            if (model.Amount < 0)
            {
                throw new Exception("Кол-во продуктов не может быть ниже нуля");
            }
            if (model.Rate < 0.0 || model.Rate > 5.0)
            {
                throw new Exception("Рейтинг продукта должен быть в пределах от 0.0 до 5.0");
            }
            _logger.LogInformation("Product. ProductName:{Name}.Price:{ Price}. Id: { Id}", model.Name, model.Price, model.Id);
            var element = _productStorage.GetElement(new ProductSearchModel
            {
                Name = model.Name
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Продукт с таким названием уже есть");
            }
        }
    }
}
